@@ -18,15 +18,15 @@
  *   Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
-
-#include "glib.h"
+#include "config.h"
 
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
 
+#include "glib.h"
+
 #include "glibintl.h"
 
 GQuark
@@ -97,6 +97,7 @@ struct _GMarkupParseContext
 
   guint document_empty : 1;
   guint parsing : 1;
+  gint balance;
 };
 
 /**
@@ -154,6 +155,8 @@ g_markup_parse_context_new (const GMarkupParser *parser,
   context->document_empty = TRUE;
   context->parsing = FALSE;
 
+  context->balance = 0;
+
   return context;
 }
 
@@ -332,7 +335,7 @@ unescape_text (GMarkupParseContext *context,
   UnescapeState state;
   const gchar *start;
 
-  str = g_string_new ("");
+  str = g_string_new (NULL);
 
   state = USTATE_INSIDE_TEXT;
   p = text;
@@ -592,9 +595,27 @@ unescape_text (GMarkupParseContext *context,
         }
     }
 
-  /* If no errors, we should have returned to USTATE_INSIDE_TEXT */
-  g_assert (context->state == STATE_ERROR ||
-            state == USTATE_INSIDE_TEXT);
+  if (context->state != STATE_ERROR) 
+    {
+      switch (state) 
+	{
+	case USTATE_INSIDE_TEXT:
+	  break;
+	case USTATE_AFTER_AMPERSAND:
+	case USTATE_INSIDE_ENTITY_NAME:
+	  set_unescape_error (context, error,
+			      NULL, NULL,
+			      G_MARKUP_ERROR_PARSE,
+			      _("Unfinished entity reference"));
+	  break;
+	case USTATE_AFTER_CHARREF_HASH:
+	  set_unescape_error (context, error,
+			      NULL, NULL,
+			      G_MARKUP_ERROR_PARSE,
+			      _("Unfinished character reference"));
+	  break;
+	}
+    }
 
   if (context->state == STATE_ERROR)
     {
@@ -617,13 +638,19 @@ advance_char (GMarkupParseContext *context)
 
   context->iter = g_utf8_next_char (context->iter);
   context->char_number += 1;
-  if (*context->iter == '\n')
-    {
-      context->line_number += 1;
-      context->char_number = 1;
-    }
 
-  return context->iter != context->current_text_end;
+  if (context->iter != context->current_text_end)
+    {
+      if (*context->iter == '\n')
+        {
+          context->line_number += 1;
+          context->char_number = 1;
+        }
+
+      return TRUE;
+    }
+  else
+    return FALSE;
 }
 
 static gboolean
@@ -660,7 +687,7 @@ add_to_partial (GMarkupParseContext *context,
                 const gchar         *text_end)
 {
   if (context->partial_chunk == NULL)
-    context->partial_chunk = g_string_new ("");
+    context->partial_chunk = g_string_new (NULL);
 
   if (text_start != text_end)
     g_string_append_len (context->partial_chunk, text_start,
@@ -704,8 +731,10 @@ find_current_text_end (GMarkupParseContext *context)
   p = context->current_text;
   next = g_utf8_find_next_char (p, end);
 
-  while (next)
+  while (next && *next)
     {
+      if (p == next)
+	next++;
       p = next;
       next = g_utf8_find_next_char (p, end);
     }
@@ -945,6 +974,7 @@ g_markup_parse_context_parse (GMarkupParseContext *context,
               const gchar *openangle = "<";
               add_to_partial (context, openangle, openangle + 1);
               context->start = context->iter;
+	      context->balance = 1;
               context->state = STATE_INSIDE_PASSTHROUGH;
             }
           else if (*context->iter == '/')
@@ -1475,13 +1505,28 @@ g_markup_parse_context_parse (GMarkupParseContext *context,
               g_free (close_name);
             }
           break;
-
+	  
         case STATE_INSIDE_PASSTHROUGH:
           /* Possible next state: AFTER_CLOSE_ANGLE */
           do
             {
-              if (*context->iter == '>')
-                break;
+	      if (*context->iter == '<') 
+		context->balance++;
+              if (*context->iter == '>') 
+		{
+		  context->balance--;
+		  add_to_partial (context, context->start, context->iter);
+		  context->start = context->iter;
+		  if ((g_str_has_prefix (context->partial_chunk->str, "<?")
+		       && g_str_has_suffix (context->partial_chunk->str, "?")) ||
+		      (g_str_has_prefix (context->partial_chunk->str, "<!--")
+		       && g_str_has_suffix (context->partial_chunk->str, "--")) ||
+		      (g_str_has_prefix (context->partial_chunk->str, "<![CDATA[") 
+		       && g_str_has_suffix (context->partial_chunk->str, "]]")) ||
+		      (g_str_has_prefix (context->partial_chunk->str, "<!DOCTYPE")
+		       && context->balance == 0)) 
+		    break;
+		}
             }
           while (advance_char (context));
 
@@ -1666,6 +1711,26 @@ g_markup_parse_context_end_parse (GMarkupParseContext *context,
   return context->state != STATE_ERROR;
 }
 
+/**
+ * g_markup_parse_context_get_element:
+ * @context: a #GMarkupParseContext
+ * @returns: the name of the currently open element, or %NULL
+ *
+ * Retrieves the name of the currently open element.
+ *
+ * Since: 2.2
+ **/
+G_CONST_RETURN gchar *
+g_markup_parse_context_get_element (GMarkupParseContext *context)
+{
+  g_return_val_if_fail (context != NULL, NULL);
+
+  if (context->tag_stack == NULL) 
+    return NULL;
+  else
+    return current_element (context);
+} 
+
 /**
  * g_markup_parse_context_get_position:
  * @context: a #GMarkupParseContext
@@ -1762,7 +1827,7 @@ g_markup_escape_text (const gchar *text,
   if (length < 0)
     length = strlen (text);
 
-  str = g_string_new ("");
+  str = g_string_new (NULL);
   append_escaped_text (str, text, length);
 
   return g_string_free (str, FALSE);
