@@ -21,19 +21,14 @@
  * Modified by the GLib Team and others 1997-2000.  See the AUTHORS
  * file for a list of people on the GLib Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
- * GLib at ftp://ftp.gtk.org/pub/gtk/.
+ * GLib at ftp://ftp.gtk.org/pub/gtk/. 
  */
 
-#if defined(G_DISABLE_SINGLE_INCLUDES) && !defined (__GLIB_H_INSIDE__) && !defined (GLIB_COMPILATION)
-#error "Only <glib.h> can be included directly."
-#endif
-
 #ifndef __G_STRING_H__
 #define __G_STRING_H__
 
 #include <glib/gtypes.h>
 #include <glib/gunicode.h>
-#include <glib/gutils.h>  /* for G_CAN_INLINE */
 
 G_BEGIN_DECLS
 
@@ -51,12 +46,8 @@ struct _GString
  */
 GStringChunk* g_string_chunk_new	   (gsize size);  
 void	      g_string_chunk_free	   (GStringChunk *chunk);
-void	      g_string_chunk_clear	   (GStringChunk *chunk);
 gchar*	      g_string_chunk_insert	   (GStringChunk *chunk,
 					    const gchar	 *string);
-gchar*	      g_string_chunk_insert_len	   (GStringChunk *chunk,
-					    const gchar	 *string,
-					    gssize        len);
 gchar*	      g_string_chunk_insert_const  (GStringChunk *chunk,
 					    const gchar	 *string);
 
@@ -109,53 +100,17 @@ GString*     g_string_insert_c          (GString	 *string,
 GString*     g_string_insert_unichar    (GString	 *string,
 					 gssize		  pos,    
 					 gunichar	  wc);
-GString*     g_string_overwrite         (GString	 *string,
-					 gsize		  pos,    
-					 const gchar	 *val);
-GString*     g_string_overwrite_len     (GString	 *string,
-					 gsize		  pos,    
-					 const gchar	 *val,
-					 gssize           len);
 GString*     g_string_erase	        (GString	 *string,
 					 gssize		  pos,
 					 gssize		  len);
 GString*     g_string_ascii_down        (GString	 *string);
 GString*     g_string_ascii_up          (GString	 *string);
-void         g_string_vprintf           (GString	 *string,
-					 const gchar	 *format,
-					 va_list          args);
 void         g_string_printf            (GString	 *string,
 					 const gchar	 *format,
 					 ...) G_GNUC_PRINTF (2, 3);
-void         g_string_append_vprintf    (GString	 *string,
-					 const gchar	 *format,
-					 va_list          args);
 void         g_string_append_printf     (GString	 *string,
 					 const gchar	 *format,
 					 ...) G_GNUC_PRINTF (2, 3);
-GString *    g_string_append_uri_escaped(GString         *string,
-					 const char      *unescaped,
-					 const char      *reserved_chars_allowed,
-					 gboolean         allow_utf8);
-
-/* -- optimize g_strig_append_c --- */
-#ifdef G_CAN_INLINE
-static inline GString*
-g_string_append_c_inline (GString *gstring,
-                          gchar    c)
-{
-  if (gstring->len + 1 < gstring->allocated_len)
-    {
-      gstring->str[gstring->len++] = c;
-      gstring->str[gstring->len] = 0;
-    }
-  else
-    g_string_insert_c (gstring, -1, c);
-  return gstring;
-}
-#define g_string_append_c(gstr,c)       g_string_append_c_inline (gstr, c)
-#endif /* G_CAN_INLINE */
-
 
 #ifndef G_DISABLE_DEPRECATED
 
@@ -176,3 +131,4 @@ GString*     g_string_up                (GString	 *string);
 G_END_DECLS
 
 #endif /* __G_STRING_H__ */
+
