@@ -18,10 +18,13 @@
  *
  * gvaluetypes.h: GLib default values
  */
+#if !defined (__GLIB_GOBJECT_H_INSIDE__) && !defined (GOBJECT_COMPILATION)
+#error "Only <glib-object.h> can be included directly."
+#endif
+
 #ifndef __G_VALUETYPES_H__
 #define __G_VALUETYPES_H__
 
-
 #include	<gobject/gvalue.h>
 
 G_BEGIN_DECLS
@@ -87,6 +90,9 @@ void		      g_value_set_pointer	(GValue	      *value,
 gpointer	      g_value_get_pointer	(const GValue *value);
 
 
+/* Convenience for registering new pointer types */
+GType                 g_pointer_type_register_static (const gchar *name);
+
 /* debugging aid, describe value contents as string */
 gchar*                g_strdup_value_contents   (const GValue *value);
 
